<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>Hello World</title>
</head>
<body>
    <h2>Hello world from <b>{{ agent.jid }}</b></h2>
    <h3>The Answer to the Ultimate Question of Life, the Universe, and Everything is:</h3>
    <h1>{{ number }}</h1>
</body>
</html>
